<?php

namespace Drupal\nft_oembed\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Url;
use Drupal\media\OEmbed\Provider;
use Drupal\media\OEmbed\Resource;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for nft_oembed module routes.
 */
class OEmbedController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The serializer.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * The custom providers info.
   *
   * @var array
   */
  protected $providers;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->httpClient = $container->get('http_client');
    $instance->request = $container->get('request_stack')->getCurrentRequest();
    // $instance->serializer = $container->get('serializer');
    $instance->providers = _nft_oembed_custom_providers_info();

    return $instance;
  }

  /**
   * Read and process requested media Url.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The list of URL scheme as JSON.
   */
  public function endpoint() {
    $query = $this->request->query->all();
    $uri = $query['url'] ?? $query['uri'] ?? NULL;
    if (!UrlHelper::isValid($uri, TRUE)) {
      throw new \Exception('Invalid URL to get OEmbed content.');
    }

    // Get clean Url object with query.
    $parsed_url = parse_url($uri);
    $parsed_parts = explode('/', $parsed_url['path']);
    parse_str($parsed_url['query'], $parsed_queries);
    $cleaned_url = $parsed_url['scheme'] . '://';
    $cleaned_url .= $parsed_url['host'];
    $cleaned_url .= $parsed_url['path'];
    $remote_url = Url::fromUri($cleaned_url, [
      'query' => $parsed_queries,
    ]);

    // Find matching provider.
    $provider = NULL;
    foreach ($this->providers as $provider_name => $provider_info) {
      $parsed_provider_url = parse_url($provider_info['provider_url']);
      if ($parsed_url['host'] == $parsed_provider_url['host']) {
        $provider = $provider_name;
        break;
      }
    }

    // Keep values as context.
    $context = [];
    $context['remote_url'] = $remote_url;
    $context['provider_id'] = $provider;
    $context['network_id'] = '1'; // mainnet (default)
    // $context['network_id'] = '4'; // rinkeby
    // $context['network_id'] = '5'; // görli
    // $context['network_id'] = '137'; // matic

    switch ($provider) {
      case 'Localhost':
        $context['contract_address'] = $parsed_parts[0];
        $context['token_id'] = $parsed_parts[1];
        break;
      case 'Etherscan':
        $context['contract_address'] = $parsed_parts[0];
        $context['token_id'] = $parsed_queries['a'];
        break;
      case 'OpenSea':
        $context['contract_address'] = $parsed_parts[2];
        $context['token_id'] = $parsed_parts[3];
        break;
      case 'Rarible':
        break;
      default;
        throw new \Exception('Error trying to identify the provider.');
    }

    // Query NFT info.
    return $this->getNft($context);

  }

  /**
   * Presents the aggregator feed creation form.
   *
   * @param string $contract_address
   *   A given contract address (must start with '0x').
   * @param string $token_id
   *   A given token ID.
   * @param string $network_id
   *   A given network ID.
   * @param string $provider_id
   *   A given provider ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The normalized content as per OEmbed specifications.
   *
   * @see https://oembed.com/#section2.3.1
   */
  public function getNft(array $context) {
    // @todo Load from cache.
    $data = [];

    // @todo Use API KEY?
    $parameters = [];

    switch ($context['provider_id']) {
      case 'OpenSea':
        // Using OpenSea.io SDK.
        // @see https://docs.opensea.io/reference/retrieving-a-single-asset
        $args = [];
        $args[] = '/asset';
        $args[] = $context['contract_address'];
        $args[] = $context['token_id'];

        $remote_url = 'https://api.opensea.io/api/v1';
        $remote_url .= implode('/', $args);
        break;
    }

    try {
      $response = $this->httpClient->get($remote_url, $parameters);
      $data = Json::decode($response->getBody()->getContents());
    } catch (Error $e) {
      $this->logger()->error($e->getMessage());
    }

    // Denormalize values.
    $denormalized_data = $this->denormalizeData($data);

    // @todo Save in cache.

    return new JsonResponse($denormalized_data);
  }

  /**
   * Standardize data to respect OEmbed specifications.
   *
   * @param array $values
   *   A given array of values.
   *
   * @return array
   *   A OEmbed-compliant list of values.
   *
   * @todo Use serializer for proper implementation.
   */
  public function denormalizeData(array $values) {
    // $class = '';
    // $format = NULL;
    // $denormalized_data = $this->serializer->denormalize($values, $class, $format, $context);

    $data = [];
    $data['type'] = Resource::TYPE_PHOTO;
    $data['version'] = '1.0';
    $data['url'] = $values['image_url'];
    $data['title'] = $values['name'];
    $data['author_name'] = $values['creator']['user']['username'];
    $data['author_url'] = $values['asset_contract']['external_link'];
    $data['thumbnail_url'] = $values['image_thumbnail_url'];
    $data['thumbnail_width'] = 100;
    $data['thumbnail_height'] = 100;

    return $data;
  }
}
